"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractDirectory = void 0;
var tslib_1 = require("tslib");
var AbstractEntry_1 = require("./AbstractEntry");
var AbstractFile_1 = require("./AbstractFile");
var errors_1 = require("./errors");
var util_1 = require("./util");
var AbstractDirectory = (function (_super) {
    (0, tslib_1.__extends)(AbstractDirectory, _super);
    function AbstractDirectory(fs, path) {
        var _a;
        var _this = _super.call(this, fs, path) || this;
        _this.ls = _this.list;
        _this.mkdir = _this.mkcol;
        _this.readdir = _this.list;
        var hook = (_a = fs.options) === null || _a === void 0 ? void 0 : _a.hook;
        if (hook) {
            _this.beforeMkcol = hook.beforeMkcol;
            _this.beforeList = hook.beforeList;
            _this.afterMkcol = hook.afterMkcol;
            _this.afterList = hook.afterList;
        }
        return _this;
    }
    AbstractDirectory.prototype._delete = function (options, errors) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var stats, e_1, children, _i, children_1, child, childEntry, e_2;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4, this.head()];
                    case 1:
                        stats = _a.sent();
                        if (stats.size != null) {
                            throw (0, errors_1.createError)({
                                name: errors_1.TypeMismatchError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: "\"" + this.path + "\" is not a directory",
                            });
                        }
                        return [3, 3];
                    case 2:
                        e_1 = _a.sent();
                        if (e_1.name === errors_1.NotFoundError.name) {
                            if (!options.force) {
                                throw e_1;
                            }
                            return [2];
                        }
                        else {
                            throw (0, errors_1.createError)({
                                name: errors_1.NotReadableError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: e_1,
                            });
                        }
                        return [3, 3];
                    case 3:
                        if (!options.recursive) return [3, 11];
                        return [4, this.list()];
                    case 4:
                        children = _a.sent();
                        _i = 0, children_1 = children;
                        _a.label = 5;
                    case 5:
                        if (!(_i < children_1.length)) return [3, 11];
                        child = children_1[_i];
                        _a.label = 6;
                    case 6:
                        _a.trys.push([6, 9, , 10]);
                        return [4, this.fs.getEntry(child)];
                    case 7:
                        childEntry = _a.sent();
                        return [4, childEntry.delete(options)];
                    case 8:
                        _a.sent();
                        return [3, 10];
                    case 9:
                        e_2 = _a.sent();
                        if (options.force) {
                            errors.push(e_2);
                        }
                        else {
                            throw e_2;
                        }
                        return [3, 10];
                    case 10:
                        _i++;
                        return [3, 5];
                    case 11: return [2, this._rmdir()];
                }
            });
        });
    };
    AbstractDirectory.prototype._xmit = function (to, copyErrors, options) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var toDir, fromPaths, _i, fromPaths_1, fromPath, toPath, stats, fromEntry, name_1, toEntry, e_3;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (to instanceof AbstractFile_1.AbstractFile) {
                            throw (0, errors_1.createError)({
                                name: errors_1.TypeMismatchError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: "\"" + this.path + "\" is not a directory",
                            });
                        }
                        toDir = to;
                        return [4, toDir.mkcol({
                                force: options.force,
                                recursive: false,
                                ignoreHook: options.ignoreHook,
                            })];
                    case 1:
                        _a.sent();
                        if (!options.recursive) {
                            return [2];
                        }
                        return [4, this.list()];
                    case 2:
                        fromPaths = _a.sent();
                        _i = 0, fromPaths_1 = fromPaths;
                        _a.label = 3;
                    case 3:
                        if (!(_i < fromPaths_1.length)) return [3, 11];
                        fromPath = fromPaths_1[_i];
                        toPath = void 0;
                        _a.label = 4;
                    case 4:
                        _a.trys.push([4, 9, , 10]);
                        return [4, this.fs.head(fromPath)];
                    case 5:
                        stats = _a.sent();
                        return [4, (stats.size != null
                                ? this.fs.getFile(fromPath)
                                : this.fs.getDirectory(fromPath))];
                    case 6:
                        fromEntry = (_a.sent());
                        name_1 = (0, util_1.getName)(fromPath);
                        toPath = (0, util_1.joinPaths)(toDir.path, name_1);
                        return [4, (stats.size != null
                                ? this.fs.getFile(toPath)
                                : this.fs.getDirectory(toPath))];
                    case 7:
                        toEntry = (_a.sent());
                        return [4, fromEntry._xmit(toEntry, copyErrors, options)];
                    case 8:
                        _a.sent();
                        return [3, 10];
                    case 9:
                        e_3 = _a.sent();
                        copyErrors.push((0, tslib_1.__assign)((0, tslib_1.__assign)({}, e_3), { from: fromPath, to: toPath }));
                        return [3, 10];
                    case 10:
                        _i++;
                        return [3, 3];
                    case 11: return [2];
                }
            });
        });
    };
    AbstractDirectory.prototype.list = function (options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var list;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(!options.ignoreHook && this.beforeList)) return [3, 2];
                        return [4, this.beforeList(this.path, options)];
                    case 1:
                        list = _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!!list) return [3, 4];
                        return [4, this._list()];
                    case 3:
                        list = _a.sent();
                        _a.label = 4;
                    case 4:
                        if (!(!options.ignoreHook && this.afterList)) return [3, 6];
                        return [4, this.afterList(this.path, list)];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6: return [2, list];
                }
            });
        });
    };
    AbstractDirectory.prototype.mkcol = function (options) {
        if (options === void 0) { options = { force: false, recursive: false }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var stats, e_4, parent_1;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 8]);
                        return [4, this.head()];
                    case 1:
                        stats = _a.sent();
                        if (stats.size != null) {
                            throw (0, errors_1.createError)({
                                name: errors_1.TypeMismatchError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: "\"" + this.path + "\" is not a directory",
                            });
                        }
                        if (!options.force) {
                            throw (0, errors_1.createError)({
                                name: errors_1.SecurityError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: "\"" + this.path + "\" has already existed",
                            });
                        }
                        return [2];
                    case 2:
                        e_4 = _a.sent();
                        if (!(e_4.name === errors_1.NotFoundError.name)) return [3, 6];
                        if (!options.recursive) return [3, 5];
                        return [4, this.getParent()];
                    case 3:
                        parent_1 = _a.sent();
                        return [4, parent_1.mkcol({ force: true, recursive: true })];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5: return [3, 7];
                    case 6: throw (0, errors_1.createError)({
                        name: errors_1.NotReadableError.name,
                        repository: this.fs.repository,
                        path: this.path,
                        e: e_4,
                    });
                    case 7: return [3, 8];
                    case 8:
                        if (!(!options.ignoreHook && this.beforeMkcol)) return [3, 10];
                        return [4, this.beforeMkcol(this.path, options)];
                    case 9:
                        if (_a.sent()) {
                            return [2];
                        }
                        _a.label = 10;
                    case 10: return [4, this._mkcol()];
                    case 11:
                        _a.sent();
                        if (!(!options.ignoreHook && this.afterMkcol)) return [3, 13];
                        return [4, this.afterMkcol(this.path)];
                    case 12:
                        _a.sent();
                        _a.label = 13;
                    case 13: return [2];
                }
            });
        });
    };
    return AbstractDirectory;
}(AbstractEntry_1.AbstractEntry));
exports.AbstractDirectory = AbstractDirectory;
//# sourceMappingURL=AbstractDirectory.js.map