"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractEntry = void 0;
var tslib_1 = require("tslib");
var util_1 = require("./util");
var AbstractEntry = (function () {
    function AbstractEntry(fs, path) {
        var _this = this;
        this.fs = fs;
        this.path = path;
        this.cp = this.copy;
        this.del = this.delete;
        this.mv = this.move;
        this.rm = this.delete;
        this.stat = this.head;
        this.patch = function (props, options) {
            if (options === void 0) { options = {}; }
            return _this.fs.patch(_this.path, props, options);
        };
        this.toString = function () { return _this.fs.repository + ":" + _this.path; };
        this.toURL = function (urlType) { return _this.fs.toURL(_this.path, urlType); };
        this.path = (0, util_1.normalizePath)(path);
        var hook = fs.options.hook;
        if (hook) {
            this.beforeDelete = hook === null || hook === void 0 ? void 0 : hook.beforeDelete;
            this.afterDelete = hook === null || hook === void 0 ? void 0 : hook.afterDelete;
        }
    }
    AbstractEntry.prototype.copy = function (to, options) {
        if (options === void 0) { options = { force: false, recursive: false }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var copyErrors;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.head()];
                    case 1:
                        _a.sent();
                        copyErrors = [];
                        return [4, this._xmit(to, copyErrors, {
                                bufferSize: options.bufferSize,
                                force: options.force,
                                recursive: options.recursive,
                            })];
                    case 2:
                        _a.sent();
                        return [2, copyErrors];
                }
            });
        });
    };
    AbstractEntry.prototype.delete = function (options) {
        if (options === void 0) { options = { force: false, recursive: false }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var errors;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(!options.ignoreHook && this.beforeDelete)) return [3, 2];
                        return [4, this.beforeDelete(this.path, options)];
                    case 1:
                        if (_a.sent()) {
                            return [2, []];
                        }
                        _a.label = 2;
                    case 2:
                        errors = [];
                        return [4, this._delete(options, errors)];
                    case 3:
                        _a.sent();
                        if (!(!options.ignoreHook && this.afterDelete)) return [3, 5];
                        return [4, this.afterDelete(this.path)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5: return [2, errors];
                }
            });
        });
    };
    AbstractEntry.prototype.getParent = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var parentPath;
            return (0, tslib_1.__generator)(this, function (_a) {
                parentPath = (0, util_1.getParentPath)(this.path);
                return [2, this.fs.getDirectory(parentPath)];
            });
        });
    };
    AbstractEntry.prototype.head = function (options) {
        if (options === void 0) { options = {}; }
        return this.fs.head(this.path, options);
    };
    AbstractEntry.prototype.move = function (to, options) {
        if (options === void 0) { options = { force: false }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var copyErrors, deleteErrors;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.head()];
                    case 1:
                        _a.sent();
                        copyErrors = [];
                        return [4, this._xmit(to, copyErrors, {
                                bufferSize: options.bufferSize,
                                force: options.force,
                                recursive: true,
                            })];
                    case 2:
                        _a.sent();
                        if (!(copyErrors.length === 0)) return [3, 4];
                        return [4, this.delete({
                                force: options.force,
                                recursive: false,
                            })];
                    case 3:
                        deleteErrors = _a.sent();
                        Array.prototype.push.apply(copyErrors, deleteErrors);
                        _a.label = 4;
                    case 4: return [2, copyErrors];
                }
            });
        });
    };
    return AbstractEntry;
}());
exports.AbstractEntry = AbstractEntry;
//# sourceMappingURL=AbstractEntry.js.map