"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractFile = void 0;
var tslib_1 = require("tslib");
var sha256_uint8array_1 = require("sha256-uint8array");
var univ_conv_1 = require("univ-conv");
var _1 = require(".");
var AbstractEntry_1 = require("./AbstractEntry");
var errors_1 = require("./errors");
var util_1 = require("./util");
var AbstractFile = (function (_super) {
    (0, tslib_1.__extends)(AbstractFile, _super);
    function AbstractFile(fs, path) {
        var _a;
        var _this = _super.call(this, fs, path) || this;
        var hook = (_a = fs.options) === null || _a === void 0 ? void 0 : _a.hook;
        if (hook) {
            _this.beforeGet = hook.beforeGet;
            _this.beforePost = hook.beforePost;
            _this.beforePut = hook.beforePut;
            _this.afterGet = hook.afterGet;
            _this.afterPost = hook.afterPost;
            _this.afterPut = hook.afterPut;
        }
        return _this;
    }
    AbstractFile.prototype._convert = function (converter, chunk, type) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var _a, base64, binaryString;
            return (0, tslib_1.__generator)(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = type;
                        switch (_a) {
                            case "ArrayBuffer": return [3, 1];
                            case "Uint8Array": return [3, 2];
                            case "Buffer": return [3, 3];
                            case "Blob": return [3, 4];
                            case "Readable": return [3, 5];
                            case "ReadableStream": return [3, 6];
                            case "Base64": return [3, 7];
                            case "BinaryString": return [3, 9];
                            case "Text": return [3, 11];
                        }
                        return [3, 12];
                    case 1: return [2, converter.toArrayBuffer(chunk)];
                    case 2: return [2, converter.toUint8Array(chunk)];
                    case 3: return [2, converter.toBuffer(chunk)];
                    case 4: return [2, converter.toBlob(chunk)];
                    case 5: return [2, converter.toReadable(chunk)];
                    case 6: return [2, converter.toReadableStream(chunk)];
                    case 7: return [4, converter.toBase64(chunk)];
                    case 8:
                        base64 = _b.sent();
                        return [2, { value: base64, encoding: "Base64" }];
                    case 9: return [4, converter.toBinaryString(chunk)];
                    case 10:
                        binaryString = _b.sent();
                        return [2, { value: binaryString, encoding: "BinaryString" }];
                    case 11: return [2, converter.toText(chunk)];
                    case 12: return [2];
                }
            });
        });
    };
    AbstractFile.prototype._delete = function (options, _errors) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var stats, e_1;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4, this.head()];
                    case 1:
                        stats = _a.sent();
                        if (stats.size == null) {
                            throw (0, errors_1.createError)({
                                name: errors_1.TypeMismatchError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: "\"" + this.path + "\" is not a file",
                            });
                        }
                        return [3, 3];
                    case 2:
                        e_1 = _a.sent();
                        if (e_1.name === errors_1.NotFoundError.name) {
                            if (!options.force) {
                                throw e_1;
                            }
                            return [2];
                        }
                        else {
                            throw (0, errors_1.createError)({
                                name: errors_1.NotReadableError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: e_1,
                            });
                        }
                        return [3, 3];
                    case 3: return [2, this._rm()];
                }
            });
        });
    };
    AbstractFile.prototype._xmit = function (toEntry, _copyErrors, options) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var to, e_2, source;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (toEntry instanceof _1.AbstractDirectory) {
                            throw (0, errors_1.createError)({
                                name: errors_1.TypeMismatchError.name,
                                repository: toEntry.fs.repository,
                                path: toEntry.path,
                                e: "\"" + toEntry + "\" is not a file",
                            });
                        }
                        to = toEntry;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4, to.head()];
                    case 2:
                        _a.sent();
                        if (!options.force) {
                            throw (0, errors_1.createError)({
                                name: _1.SecurityError.name,
                                repository: to.fs.repository,
                                path: to.path,
                            });
                        }
                        return [3, 4];
                    case 3:
                        e_2 = _a.sent();
                        if (e_2.name !== errors_1.NotFoundError.name) {
                            throw (0, errors_1.createError)({
                                name: errors_1.NotReadableError.name,
                                repository: to.fs.repository,
                                path: to.path,
                                e: e_2,
                            });
                        }
                        return [3, 4];
                    case 4: return [4, this.getSource(options)];
                    case 5:
                        source = _a.sent();
                        return [4, to.write(source, { bufferSize: options.bufferSize })];
                    case 6:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    AbstractFile.prototype.hash = function (options) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var converter, source, streamSource, hash;
            var _this = this;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        options = options || {};
                        converter = this._getConverter(options.bufferSize);
                        return [4, this.getSource(options)];
                    case 1:
                        source = _a.sent();
                        return [4, converter.toStreamSource(source)];
                    case 2:
                        streamSource = _a.sent();
                        hash = (0, sha256_uint8array_1.createHash)();
                        return [4, (0, univ_conv_1.handleStreamSource)(streamSource, function (chunk) { return (0, tslib_1.__awaiter)(_this, void 0, void 0, function () {
                                var buffer;
                                return (0, tslib_1.__generator)(this, function (_a) {
                                    switch (_a.label) {
                                        case 0: return [4, converter.toUint8Array(chunk)];
                                        case 1:
                                            buffer = _a.sent();
                                            hash.update(buffer);
                                            return [2];
                                    }
                                });
                            }); })];
                    case 3:
                        _a.sent();
                        return [2, (0, util_1.toHex)(hash.digest())];
                }
            });
        });
    };
    AbstractFile.prototype.read = function (options) {
        var _a;
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var type, source, converter;
            return (0, tslib_1.__generator)(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        type = (_a = options === null || options === void 0 ? void 0 : options.sourceType) !== null && _a !== void 0 ? _a : (univ_conv_1.isBrowser ? "Blob" : "Uint8Array");
                        options = { sourceType: type };
                        return [4, this.getSource(options)];
                    case 1:
                        source = _b.sent();
                        converter = this._getConverter(options === null || options === void 0 ? void 0 : options.bufferSize);
                        return [2, this._convert(converter, source, type)];
                }
            });
        });
    };
    AbstractFile.prototype.write = function (src, options) {
        var _a;
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var path, fs, repository, create, e_3;
            return (0, tslib_1.__generator)(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        path = this.path;
                        fs = this.fs;
                        repository = fs.repository;
                        _b.label = 1;
                    case 1:
                        _b.trys.push([1, 3, , 4]);
                        return [4, this.head()];
                    case 2:
                        _b.sent();
                        if (options === null || options === void 0 ? void 0 : options.create) {
                            throw (0, errors_1.createError)({
                                name: _1.PathExistError.name,
                                repository: repository,
                                path: path,
                            });
                        }
                        create = false;
                        return [3, 4];
                    case 3:
                        e_3 = _b.sent();
                        if (e_3.name === errors_1.NotFoundError.name) {
                            if ((options === null || options === void 0 ? void 0 : options.create) === false) {
                                throw (0, errors_1.createError)({
                                    name: errors_1.NotFoundError.name,
                                    repository: repository,
                                    path: path,
                                    e: e_3,
                                });
                            }
                            create = true;
                        }
                        else {
                            throw (0, errors_1.createError)({
                                name: errors_1.NotReadableError.name,
                                repository: repository,
                                path: path,
                                e: e_3,
                            });
                        }
                        return [3, 4];
                    case 4:
                        options = { append: (_a = options === null || options === void 0 ? void 0 : options.append) !== null && _a !== void 0 ? _a : false, create: create };
                        if (!create) return [3, 7];
                        if (!this.beforePost) return [3, 6];
                        return [4, this.beforePost(path, src, options)];
                    case 5:
                        if (_b.sent()) {
                            return [2];
                        }
                        _b.label = 6;
                    case 6: return [3, 9];
                    case 7:
                        if (!this.beforePut) return [3, 9];
                        return [4, this.beforePut(path, src, options)];
                    case 8:
                        if (_b.sent()) {
                            return [2];
                        }
                        _b.label = 9;
                    case 9: return [4, this._write(src, options)];
                    case 10:
                        _b.sent();
                        if (create) {
                            if (this.afterPost) {
                                this.afterPost(path, src).catch(function (e) { return console.warn(e); });
                            }
                        }
                        else {
                            if (this.afterPut) {
                                this.afterPut(path, src).catch(function (e) { return console.warn(e); });
                            }
                        }
                        return [2];
                }
            });
        });
    };
    AbstractFile.prototype._getConverter = function (bufferSize) {
        return bufferSize ? new univ_conv_1.Converter({ bufferSize: bufferSize }) : univ_conv_1.converter;
    };
    AbstractFile.prototype.getSource = function (options) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var ignoreHook, path, source;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        ignoreHook = options.ignoreHook;
                        path = this.path;
                        source = null;
                        if (!(!ignoreHook && this.beforeGet)) return [3, 2];
                        return [4, this.beforeGet(path, options)];
                    case 1:
                        source = _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!!source) return [3, 4];
                        return [4, this._getSource(options)];
                    case 3:
                        source = _a.sent();
                        _a.label = 4;
                    case 4:
                        if (!ignoreHook && this.afterGet) {
                            this.afterGet(path, source).catch(function (e) { return console.warn(e); });
                        }
                        return [2, source];
                }
            });
        });
    };
    return AbstractFile;
}(AbstractEntry_1.AbstractEntry));
exports.AbstractFile = AbstractFile;
//# sourceMappingURL=AbstractFile.js.map