import { Source } from "univ-conv";
import { ReadOptions, CopyOptions, DeleteOptions, Directory, Entry, ErrorLike, File, FileSystem, FileSystemOptions, HeadOptions, ListOptions, MkcolOptions, MoveOptions, OpenOptions, PatchOptions, Props, Stats, URLType, WriteOptions } from "./core";
export declare abstract class AbstractFileSystem implements FileSystem {
    readonly repository: string;
    readonly options: FileSystemOptions;
    private afterHead?;
    private afterPatch?;
    private beforeHead?;
    private beforePatch?;
    cp: (fromPath: string, toPath: string, options?: CopyOptions) => Promise<ErrorLike[]>;
    del: (path: string, options?: DeleteOptions) => Promise<ErrorLike[]>;
    ls: (path: string, options?: ListOptions | undefined) => Promise<string[]>;
    mkdir: (path: string, options?: MkcolOptions | undefined) => Promise<void>;
    mv: (fromPath: string, toPath: string, options?: MoveOptions) => Promise<ErrorLike[]>;
    readdir: (path: string, options?: ListOptions | undefined) => Promise<string[]>;
    rm: (path: string, options?: DeleteOptions) => Promise<ErrorLike[]>;
    stat: (path: string, options?: HeadOptions) => Promise<Stats>;
    unlink: (path: string, options?: DeleteOptions) => Promise<ErrorLike[]>;
    constructor(repository: string, options?: FileSystemOptions);
    copy(fromPath: string, toPath: string, options?: CopyOptions): Promise<ErrorLike[]>;
    delete(path: string, options?: DeleteOptions): Promise<ErrorLike[]>;
    getEntry(path: string): Promise<Entry>;
    hash(path: string, options?: OpenOptions): Promise<string>;
    head(path: string, options?: HeadOptions): Promise<Stats>;
    list(path: string, options?: ListOptions): Promise<string[]>;
    mkcol(path: string, options?: MkcolOptions): Promise<void>;
    move(fromPath: string, toPath: string, options?: MoveOptions): Promise<ErrorLike[]>;
    patch(path: string, props: Props, options?: PatchOptions): Promise<void>;
    read(path: string, options?: ReadOptions): Promise<Source>;
    write(path: string, source: Source, options?: WriteOptions): Promise<void>;
    abstract _head(path: string, options: HeadOptions): Promise<Stats>;
    abstract _patch(path: string, props: Props, options: PatchOptions): Promise<void>;
    abstract getDirectory(path: string): Promise<Directory>;
    abstract getFile(path: string): Promise<File>;
    abstract toURL(path: string, urlType?: URLType): Promise<string>;
    private _prepareXmit;
}
//# sourceMappingURL=AbstractFileSystem.d.ts.map