"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractFileSystem = void 0;
var tslib_1 = require("tslib");
var AbstractFile_1 = require("./AbstractFile");
var errors_1 = require("./errors");
var util_1 = require("./util");
var AbstractFileSystem = (function () {
    function AbstractFileSystem(repository, options) {
        if (options === void 0) { options = {}; }
        this.repository = repository;
        this.options = options;
        this.cp = this.copy;
        this.del = this.delete;
        this.ls = this.list;
        this.mkdir = this.mkcol;
        this.mv = this.move;
        this.readdir = this.list;
        this.rm = this.delete;
        this.stat = this.head;
        this.unlink = this.delete;
        var hook = options.hook;
        this.beforeHead = hook === null || hook === void 0 ? void 0 : hook.beforeHead;
        this.beforePatch = hook === null || hook === void 0 ? void 0 : hook.beforePatch;
        this.afterHead = hook === null || hook === void 0 ? void 0 : hook.afterHead;
        this.afterPatch = hook === null || hook === void 0 ? void 0 : hook.afterPatch;
    }
    AbstractFileSystem.prototype.copy = function (fromPath, toPath, options) {
        if (options === void 0) { options = { force: false, recursive: false }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var _a, from, to;
            return (0, tslib_1.__generator)(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4, this._prepareXmit(fromPath, toPath)];
                    case 1:
                        _a = _b.sent(), from = _a.from, to = _a.to;
                        return [2, from.copy(to, options)];
                }
            });
        });
    };
    AbstractFileSystem.prototype.delete = function (path, options) {
        if (options === void 0) { options = { force: false, recursive: false }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var entry;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.getEntry(path)];
                    case 1:
                        entry = _a.sent();
                        return [2, entry.delete(options)];
                }
            });
        });
    };
    AbstractFileSystem.prototype.getEntry = function (path) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var stats;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.head(path)];
                    case 1:
                        stats = _a.sent();
                        return [2, stats.size != null ? this.getFile(path) : this.getDirectory(path)];
                }
            });
        });
    };
    AbstractFileSystem.prototype.hash = function (path, options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var file;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.getFile(path)];
                    case 1:
                        file = _a.sent();
                        return [2, file.hash(options)];
                }
            });
        });
    };
    AbstractFileSystem.prototype.head = function (path, options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var stats;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        path = (0, util_1.normalizePath)(path);
                        if (!(!options.ignoreHook && this.beforeHead)) return [3, 2];
                        return [4, this.beforeHead(path, options)];
                    case 1:
                        stats = _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!!stats) return [3, 4];
                        return [4, this._head(path, options)];
                    case 3:
                        stats = _a.sent();
                        _a.label = 4;
                    case 4:
                        if (this.options.logicalDelete && stats.deleted != null) {
                            throw (0, errors_1.createError)({
                                repository: this.repository,
                                path: path,
                                e: undefined,
                                name: errors_1.NotFoundError.name,
                            });
                        }
                        if (!(!options.ignoreHook && this.afterHead)) return [3, 6];
                        return [4, this.afterHead(path, stats)];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6: return [2, stats];
                }
            });
        });
    };
    AbstractFileSystem.prototype.list = function (path, options) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var dir;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.getDirectory(path)];
                    case 1:
                        dir = _a.sent();
                        return [2, dir.list(options)];
                }
            });
        });
    };
    AbstractFileSystem.prototype.mkcol = function (path, options) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var dir;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.getDirectory(path)];
                    case 1:
                        dir = _a.sent();
                        return [2, dir.mkcol(options)];
                }
            });
        });
    };
    AbstractFileSystem.prototype.move = function (fromPath, toPath, options) {
        if (options === void 0) { options = { force: false }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var _a, from, to;
            return (0, tslib_1.__generator)(this, function (_b) {
                switch (_b.label) {
                    case 0: return [4, this._prepareXmit(fromPath, toPath)];
                    case 1:
                        _a = _b.sent(), from = _a.from, to = _a.to;
                        return [2, from.move(to, options)];
                }
            });
        });
    };
    AbstractFileSystem.prototype.patch = function (path, props, options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var stats;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        path = (0, util_1.normalizePath)(path);
                        if (!this.beforePatch) return [3, 2];
                        return [4, this.beforePatch(path, props, options)];
                    case 1:
                        if (_a.sent()) {
                            return [2];
                        }
                        _a.label = 2;
                    case 2: return [4, this.head(path, { ignoreHook: options.ignoreHook })];
                    case 3:
                        stats = _a.sent();
                        props = (0, tslib_1.__assign)((0, tslib_1.__assign)({}, stats), props);
                        return [4, this._patch(path, props, options)];
                    case 4:
                        _a.sent();
                        if (!this.afterPatch) return [3, 6];
                        return [4, this.afterPatch(path)];
                    case 5:
                        _a.sent();
                        _a.label = 6;
                    case 6: return [2];
                }
            });
        });
    };
    AbstractFileSystem.prototype.read = function (path, options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var file;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.getFile(path)];
                    case 1:
                        file = _a.sent();
                        return [2, file.read(options)];
                }
            });
        });
    };
    AbstractFileSystem.prototype.write = function (path, source, options) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var file;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.getFile(path)];
                    case 1:
                        file = _a.sent();
                        return [4, file.write(source, options)];
                    case 2:
                        _a.sent();
                        return [2];
                }
            });
        });
    };
    AbstractFileSystem.prototype._prepareXmit = function (fromPath, toPath) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var from, to;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.getEntry(fromPath)];
                    case 1:
                        from = _a.sent();
                        return [4, (from instanceof AbstractFile_1.AbstractFile
                                ? this.getFile(toPath)
                                : this.getDirectory(toPath))];
                    case 2:
                        to = _a.sent();
                        return [2, { from: from, to: to }];
                }
            });
        });
    };
    return AbstractFileSystem;
}());
exports.AbstractFileSystem = AbstractFileSystem;
//# sourceMappingURL=AbstractFileSystem.js.map