"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createError = exports.domExceptions = exports.PathExistError = exports.NotAllowedError = exports.OperationError = exports.VersionError = exports.ReadOnlyError = exports.TransactionInactiveError = exports.DataError = exports.ConstraintError = exports.UnknownError = exports.NotReadableError = exports.EncodingError = exports.DataCloneError = exports.InvalidNodeTypeError = exports.TimeoutError = exports.QuotaExceededError = exports.URLMismatchError = exports.AbortError = exports.NetworkError = exports.SecurityError = exports.TypeMismatchError = exports.InvalidAccessError = exports.NamespaceError = exports.InvalidModificationError = exports.SyntaxError = exports.InvalidStateError = exports.InUseAttributeError = exports.NotSupportedError = exports.NotFoundError = exports.NoModificationAllowedError = exports.InvalidCharacterError = exports.WrongDocumentError = exports.HierarchyRequestError = exports.IndexSizeError = void 0;
var tslib_1 = require("tslib");
exports.IndexSizeError = {
    code: 1,
    name: "IndexSizeError",
    message: "The index is not in the allowed range.",
};
exports.HierarchyRequestError = {
    code: 3,
    name: "HierarchyRequestError",
    message: "The operation would yield an incorrect node tree. [DOM]",
};
exports.WrongDocumentError = {
    code: 4,
    name: "WrongDocumentError",
    message: "The object is in the wrong document. [DOM]",
};
exports.InvalidCharacterError = {
    code: 5,
    name: "InvalidCharacterError",
    message: "The string contains invalid characters.",
};
exports.NoModificationAllowedError = {
    code: 7,
    name: "NoModificationAllowedError",
    message: "The string contains invalid characters.",
};
exports.NotFoundError = {
    code: 8,
    name: "NotFoundError",
    message: "The object can not be found here.",
};
exports.NotSupportedError = {
    code: 9,
    name: "NotSupportedError",
    message: "The operation is not supported.",
};
exports.InUseAttributeError = {
    code: 10,
    name: "InUseAttributeError",
    message: "The attribute is in use.",
};
exports.InvalidStateError = {
    code: 11,
    name: "InvalidStateError",
    message: "The object is in an invalid state.",
};
exports.SyntaxError = {
    code: 12,
    name: "SyntaxError",
    message: "The string did not match the expected pattern.",
};
exports.InvalidModificationError = {
    code: 13,
    name: "InvalidModificationError",
    message: "The object can not be modified in this way.",
};
exports.NamespaceError = {
    code: 14,
    name: "NamespaceError",
    message: "The operation is not allowed by Namespaces in XML. [XML-NAMES]",
};
exports.InvalidAccessError = {
    code: 15,
    name: "InvalidAccessError",
    message: "The object does not support the operation or argument.",
};
exports.TypeMismatchError = {
    code: 17,
    name: "TypeMismatchError",
    message: "The type of the object does not match the expected type.",
};
exports.SecurityError = {
    code: 18,
    name: "SecurityError",
    message: "The operation is insecure.",
};
exports.NetworkError = {
    code: 19,
    name: "NetworkError",
    message: "A network error occurred.",
};
exports.AbortError = {
    code: 20,
    name: "AbortError",
    message: "The operation was aborted.",
};
exports.URLMismatchError = {
    code: 21,
    name: "URLMismatchError",
    message: "The given URL does not match another URL.",
};
exports.QuotaExceededError = {
    code: 22,
    name: "QuotaExceededError",
    message: "The quota has been exceeded.",
};
exports.TimeoutError = {
    code: 23,
    name: "TimeoutError",
    message: "The operation timed out.",
};
exports.InvalidNodeTypeError = {
    code: 24,
    name: "InvalidNodeTypeError",
    message: "The supplied node is incorrect or has an incorrect ancestor for this operation.",
};
exports.DataCloneError = {
    code: 25,
    name: "DataCloneError",
    message: "The object can not be cloned.",
};
exports.EncodingError = {
    name: "EncodingError",
    message: "The encoding operation (either encoded or decoding) failed.",
};
exports.NotReadableError = {
    name: "NotReadableError",
    message: "The I/O read operation failed.",
};
exports.UnknownError = {
    name: "UnknownError",
    message: "The operation failed for an unknown transient reason.",
};
exports.ConstraintError = {
    name: "ConstraintError",
    message: "A mutation operation in a transaction failed because a constraint was not satisfied.",
};
exports.DataError = {
    name: "DataError",
    message: "Provided data is inadequate.",
};
exports.TransactionInactiveError = {
    name: "TransactionInactiveError",
    message: "A request was placed against a transaction which is currently not active, or which is finished.",
};
exports.ReadOnlyError = {
    name: "ReadOnlyError",
    message: 'The mutating operation was attempted in a "readonly" transaction.',
};
exports.VersionError = {
    name: "VersionError",
    message: "An attempt was made to open a database using a lower version than the existing version.",
};
exports.OperationError = {
    name: "OperationError",
    message: "The operation failed for an operation-specific reason.",
};
exports.NotAllowedError = {
    name: "NotAllowedError",
    message: "The request is not allowed by the user agent or the platform in the current context, possibly because the user denied permission.",
};
exports.PathExistError = {
    code: 12,
    name: "PathExistError",
    message: "The request file or directry has already existed.",
};
exports.domExceptions = [
    exports.IndexSizeError,
    exports.HierarchyRequestError,
    exports.WrongDocumentError,
    exports.InvalidCharacterError,
    exports.NoModificationAllowedError,
    exports.NotFoundError,
    exports.NotSupportedError,
    exports.InUseAttributeError,
    exports.InvalidStateError,
    exports.SyntaxError,
    exports.InvalidModificationError,
    exports.NamespaceError,
    exports.InvalidAccessError,
    exports.TypeMismatchError,
    exports.SecurityError,
    exports.NetworkError,
    exports.AbortError,
    exports.URLMismatchError,
    exports.QuotaExceededError,
    exports.TimeoutError,
    exports.InvalidNodeTypeError,
    exports.DataCloneError,
    exports.EncodingError,
    exports.NotReadableError,
    exports.UnknownError,
    exports.ConstraintError,
    exports.DataError,
    exports.TransactionInactiveError,
    exports.ReadOnlyError,
    exports.VersionError,
    exports.OperationError,
    exports.NotAllowedError,
];
function createError(options) {
    var e = options.e;
    if (typeof e === "object") {
        if (Object.isFrozen(e) || Object.isSealed(e)) {
            e = (0, tslib_1.__assign)({}, e);
        }
    }
    else {
        e = { message: e };
    }
    var repository = options.repository;
    if (repository.endsWith("/")) {
        repository = repository.substr(0, repository.length - 1);
    }
    if (!repository.startsWith("/")) {
        repository = "/" + repository;
    }
    var path = options.path;
    e.repository = repository;
    e.path = path;
    var name = options.name;
    if (name) {
        for (var _i = 0, domExceptions_1 = exports.domExceptions; _i < domExceptions_1.length; _i++) {
            var de = domExceptions_1[_i];
            if (de.name == name) {
                e.name = name;
                e.code = de.code;
                if (!e.message) {
                    e.message = de.message;
                }
                return e;
            }
        }
    }
    return e;
}
exports.createError = createError;
//# sourceMappingURL=errors.js.map