"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toHex = exports.isIllegalFileName = exports.normalizePath = exports.joinPaths = exports.getName = exports.getParentPath = exports.getPathParts = void 0;
var tslib_1 = require("tslib");
var errors_1 = require("./errors");
function getPathParts(path) {
    var parts = path.split("/");
    var pathParts = [];
    for (var _i = 0, parts_1 = parts; _i < parts_1.length; _i++) {
        var part = parts_1[_i];
        if (part === "..") {
            if (pathParts.length === 0) {
                throw (0, errors_1.createError)({
                    name: errors_1.SyntaxError.name,
                    repository: "",
                    path: path,
                });
            }
            pathParts.pop();
        }
        else if (part === ".") {
        }
        else if (part !== "") {
            pathParts.push(part);
        }
    }
    return pathParts;
}
exports.getPathParts = getPathParts;
function getParentPath(path) {
    var parts = getPathParts(path);
    if (parts.length <= 1) {
        return "/";
    }
    parts = parts.slice(0, -1);
    return "/" + parts.join("/");
}
exports.getParentPath = getParentPath;
function getName(path) {
    var parts = getPathParts(path);
    if (parts.length === 0) {
        return "";
    }
    return parts[parts.length - 1];
}
exports.getName = getName;
function joinPaths(path1, path2) {
    var parts1 = getPathParts(path1);
    var parts2 = getPathParts(path2);
    var parts = (0, tslib_1.__spreadArray)((0, tslib_1.__spreadArray)([], parts1, true), parts2, true);
    return "/" + parts.join("/");
}
exports.joinPaths = joinPaths;
function normalizePath(path) {
    var parts = getPathParts(path);
    return "/" + parts.join("/");
}
exports.normalizePath = normalizePath;
function isIllegalFileName(name) {
    return /[\x00-\x1f\x7f-\x9f\\/:*?"<>|]/.test(name);
}
exports.isIllegalFileName = isIllegalFileName;
var LUT_HEX_4b = new Array(0x10);
for (var n = 0; n < 0x10; n++) {
    LUT_HEX_4b[n] = n.toString(16);
}
var LUT_HEX_8b = new Array(0x100);
for (var n = 0; n < 0x100; n++) {
    LUT_HEX_8b[n] = "" + LUT_HEX_4b[(n >>> 4) & 0xf] + LUT_HEX_4b[n & 0xf];
}
function toHex(u8) {
    return u8.reduce(function (result, i) { return result + LUT_HEX_8b[i]; }, "");
}
exports.toHex = toHex;
//# sourceMappingURL=util.js.map